@@ -79,12 +79,15 @@ TrackerPropertyType tracker_property_get_data_type        (TrackerProperty
 TrackerClass *      tracker_property_get_domain           (TrackerProperty      *property);
 TrackerClass *      tracker_property_get_range            (TrackerProperty      *property);
 gint                tracker_property_get_weight           (TrackerProperty      *property);
+gint                tracker_property_get_id               (TrackerProperty      *property);
 gboolean            tracker_property_get_indexed          (TrackerProperty      *property);
 gboolean            tracker_property_get_fulltext_indexed (TrackerProperty      *property);
+gboolean            tracker_property_get_fulltext_no_limit(TrackerProperty      *property);
 gboolean            tracker_property_get_embedded         (TrackerProperty      *property);
 gboolean            tracker_property_get_multiple_values  (TrackerProperty      *property);
 gboolean            tracker_property_get_filtered         (TrackerProperty      *property);
 gboolean            tracker_property_get_transient        (TrackerProperty      *property);
+gboolean            tracker_property_get_is_new           (TrackerProperty      *property);
 gboolean            tracker_property_get_is_inverse_functional_property
 (TrackerProperty      *property);
 TrackerProperty **  tracker_property_get_super_properties (TrackerProperty      *property);
@@ -98,10 +101,14 @@ void                tracker_property_set_range            (TrackerProperty
                                                            TrackerClass         *range);
 void                tracker_property_set_weight           (TrackerProperty      *property,
                                                            gint                  value);
+void                tracker_property_set_id               (TrackerProperty      *property,
+                                                           gint                  value);
 void                tracker_property_set_indexed          (TrackerProperty      *property,
                                                            gboolean              value);
 void                tracker_property_set_fulltext_indexed (TrackerProperty      *property,
                                                            gboolean              value);
+void                tracker_property_set_fulltext_no_limit(TrackerProperty      *property,
+                                                           gboolean              value);
 void                tracker_property_set_embedded         (TrackerProperty      *property,
                                                            gboolean              value);
 void                tracker_property_set_multiple_values  (TrackerProperty      *property,
@@ -110,6 +117,8 @@ void                tracker_property_set_filtered         (TrackerProperty
                                                            gboolean              value);
 void                tracker_property_set_transient        (TrackerProperty      *property,
                                                            gboolean              value);
+void                tracker_property_set_is_new           (TrackerProperty      *property,
+                                                           gboolean              value);
 void                tracker_property_set_is_inverse_functional_property
 (TrackerProperty      *property,
  gboolean              value);
