@@ -25,9 +25,9 @@
 #include <time.h>
 
 #include <libtracker-common/tracker-common.h>
-#include <libtracker-common/tracker-type-utils.h>
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 
 #include <libtracker-fts/tracker-fts.h>
 
@@ -72,14 +72,16 @@ struct _TrackerDataUpdateBufferResource {
 	GHashTable *predicates;
 	/* string -> TrackerDataUpdateBufferTable */
 	GHashTable *tables;
-	/* string */
+	/* TrackerClass */
 	GPtrArray *types;
 };
 
 struct _TrackerDataUpdateBufferProperty {
 	gchar *name;
 	GValue value;
-	gboolean fts;
+	gchar *graph;
+	gboolean fts : 1;
+	gboolean date_time : 1;
 };
 
 struct _TrackerDataUpdateBufferTable {
@@ -197,10 +199,10 @@ GQuark tracker_data_error_quark (void) {
 }
 
 static gint
-tracker_data_update_get_new_service_id (TrackerDBInterface *iface)
+tracker_data_update_get_new_service_id (void)
 {
 	TrackerDBCursor    *cursor;
-	TrackerDBInterface *temp_iface;
+	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
 
 	static gint         max = 0;
@@ -209,9 +211,9 @@ tracker_data_update_get_new_service_id (TrackerDBInterface *iface)
 		return ++max;
 	}
 
-	temp_iface = tracker_db_manager_get_db_interface ();
+	iface = tracker_db_manager_get_db_interface ();
 
-	stmt = tracker_db_interface_create_statement (temp_iface,
+	stmt = tracker_db_interface_create_statement (iface,
 	                                              "SELECT MAX(ID) AS A FROM Resource");
 	cursor = tracker_db_statement_start_cursor (stmt, NULL);
 	g_object_unref (stmt);
@@ -276,6 +278,7 @@ cache_table_free (TrackerDataUpdateBufferTable *table)
 		property = &g_array_index (table->properties, TrackerDataUpdateBufferProperty, i);
 		g_free (property->name);
 		g_value_unset (&property->value);
+		g_free (property->graph);
 	}
 
 	g_array_free (table->properties, TRUE);
@@ -312,15 +315,19 @@ static void
 cache_insert_value (const gchar            *table_name,
                     const gchar            *field_name,
                     GValue                 *value,
+                    const gchar            *graph,
                     gboolean                multiple_values,
-                    gboolean                fts)
+                    gboolean                fts,
+                    gboolean                date_time)
 {
 	TrackerDataUpdateBufferTable    *table;
 	TrackerDataUpdateBufferProperty  property;
 
 	property.name = g_strdup (field_name);
 	property.value = *value;
+	property.graph = g_strdup (graph);
 	property.fts = fts;
+	property.date_time = date_time;
 
 	table = cache_ensure_table (table_name, multiple_values);
 	g_array_append_val (table->properties, property);
@@ -341,14 +348,17 @@ cache_delete_value (const gchar            *table_name,
                     const gchar            *field_name,
                     GValue                 *value,
                     gboolean                multiple_values,
-                    gboolean                fts)
+                    gboolean                fts,
+                    gboolean                date_time)
 {
 	TrackerDataUpdateBufferTable    *table;
 	TrackerDataUpdateBufferProperty  property;
 
 	property.name = g_strdup (field_name);
 	property.value = *value;
+	property.graph = NULL;
 	property.fts = fts;
+	property.date_time = date_time;
 
 	table = cache_ensure_table (table_name, multiple_values);
 	table->delete_value = TRUE;
@@ -377,7 +387,7 @@ static gint
 ensure_resource_id (const gchar *uri,
                     gboolean    *create)
 {
-	TrackerDBInterface *iface, *common;
+	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
 
 	gint id;
@@ -389,11 +399,9 @@ ensure_resource_id (const gchar *uri,
 	}
 
 	if (id == 0) {
-		/* object resource not yet in the database */
-		common = tracker_db_manager_get_db_interface ();
 		iface = tracker_db_manager_get_db_interface ();
 
-		id = tracker_data_update_get_new_service_id (common);
+		id = tracker_data_update_get_new_service_id ();
 		stmt = tracker_db_interface_create_statement (iface, "INSERT INTO Resource (ID, Uri) VALUES (?, ?)");
 		tracker_db_statement_bind_int (stmt, 0, id);
 		tracker_db_statement_bind_text (stmt, 1, uri);
@@ -412,24 +420,33 @@ ensure_resource_id (const gchar *uri,
 
 static void
 statement_bind_gvalue (TrackerDBStatement *stmt,
-                       gint                idx,
+                       gint               *idx,
                        const GValue       *value)
 {
-	switch (G_VALUE_TYPE (value)) {
+	GType type;
+
+	type = G_VALUE_TYPE (value);
+	switch (type) {
 	case G_TYPE_STRING:
-		tracker_db_statement_bind_text (stmt, idx, g_value_get_string (value));
+		tracker_db_statement_bind_text (stmt, (*idx)++, g_value_get_string (value));
 		break;
 	case G_TYPE_INT:
-		tracker_db_statement_bind_int (stmt, idx, g_value_get_int (value));
+		tracker_db_statement_bind_int (stmt, (*idx)++, g_value_get_int (value));
 		break;
 	case G_TYPE_INT64:
-		tracker_db_statement_bind_int64 (stmt, idx, g_value_get_int64 (value));
+		tracker_db_statement_bind_int64 (stmt, (*idx)++, g_value_get_int64 (value));
 		break;
 	case G_TYPE_DOUBLE:
-		tracker_db_statement_bind_double (stmt, idx, g_value_get_double (value));
+		tracker_db_statement_bind_double (stmt, (*idx)++, g_value_get_double (value));
 		break;
 	default:
-		g_warning ("Unknown type for binding: %s\n", G_VALUE_TYPE_NAME (value));
+		if (type == TRACKER_TYPE_DATE_TIME) {
+			tracker_db_statement_bind_int64 (stmt, (*idx)++, tracker_date_time_get_time (value));
+			tracker_db_statement_bind_int (stmt, (*idx)++, tracker_date_time_get_local_date (value));
+			tracker_db_statement_bind_int (stmt, (*idx)++, tracker_date_time_get_local_time (value));
+		} else {
+			g_warning ("Unknown type for binding: %s\n", G_VALUE_TYPE_NAME (value));
+		}
 		break;
 	}
 }
@@ -444,7 +461,7 @@ tracker_data_resource_buffer_flush (GError **error)
 	GHashTableIter                  iter;
 	const gchar                    *table_name;
 	GString                        *sql, *fts;
-	int                             i;
+	gint                            i, param;
 	GError                         *actual_error = NULL;
 
 	iface = tracker_db_manager_get_db_interface ();
@@ -479,15 +496,33 @@ tracker_data_resource_buffer_flush (GError **error)
 					                                              "DELETE FROM \"%s\" WHERE ID = ? AND \"%s\" = ?",
 					                                              table_name,
 					                                              property->name);
+				} else if (property->date_time) {
+					stmt = tracker_db_interface_create_statement (iface,
+					                                              "INSERT OR IGNORE INTO \"%s\" (ID, \"%s\", \"%s:localDate\", \"%s:localTime\", \"%s:graph\") VALUES (?, ?, ?, ?, ?)",
+					                                              table_name,
+					                                              property->name,
+					                                              property->name,
+					                                              property->name,
+					                                              property->name);
 				} else {
 					stmt = tracker_db_interface_create_statement (iface,
-					                                              "INSERT OR IGNORE INTO \"%s\" (ID, \"%s\") VALUES (?, ?)",
+					                                              "INSERT OR IGNORE INTO \"%s\" (ID, \"%s\", \"%s:graph\") VALUES (?, ?, ?)",
 					                                              table_name,
+					                                              property->name,
 					                                              property->name);
 				}
 
-				tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
-				statement_bind_gvalue (stmt, 1, &property->value);
+				param = 0;
+
+				tracker_db_statement_bind_int (stmt, param++, resource_buffer->id);
+				statement_bind_gvalue (stmt, &param, &property->value);
+
+				if (property->graph) {
+					tracker_db_statement_bind_int (stmt, param++, ensure_resource_id (property->graph, NULL));
+				} else {
+					tracker_db_statement_bind_null (stmt, param++);
+				}
+
 				tracker_db_statement_execute (stmt, &actual_error);
 				g_object_unref (stmt);
 
@@ -569,23 +604,42 @@ tracker_data_resource_buffer_flush (GError **error)
 					g_string_append (sql, ", ");
 				}
 				g_string_append_printf (sql, "\"%s\" = ?", property->name);
+
+				if (property->date_time) {
+					g_string_append_printf (sql, ", \"%s:localDate\" = ?", property->name);
+					g_string_append_printf (sql, ", \"%s:localTime\" = ?", property->name);
+				}
+
+				g_string_append_printf (sql, ", \"%s:graph\" = ?", property->name);
 			}
 
 			g_string_append (sql, " WHERE ID = ?");
 
 			stmt = tracker_db_interface_create_statement (iface, "%s", sql->str);
-			tracker_db_statement_bind_int (stmt, i, resource_buffer->id);
 
+			param = 0;
 			for (i = 0; i < table->properties->len; i++) {
 				property = &g_array_index (table->properties, TrackerDataUpdateBufferProperty, i);
 				if (table->delete_value) {
 					/* just set value to NULL for single value properties */
-					tracker_db_statement_bind_null (stmt, i);
+					tracker_db_statement_bind_null (stmt, param++);
+					if (property->date_time) {
+						/* also set localDate and localTime to NULL */
+						tracker_db_statement_bind_null (stmt, param++);
+						tracker_db_statement_bind_null (stmt, param++);
+					}
 				} else {
-					statement_bind_gvalue (stmt, i, &property->value);
+					statement_bind_gvalue (stmt, &param, &property->value);
+				}
+				if (property->graph) {
+					tracker_db_statement_bind_int (stmt, param++, ensure_resource_id (property->graph, NULL));
+				} else {
+					tracker_db_statement_bind_null (stmt, param++);
 				}
 			}
 
+			tracker_db_statement_bind_int (stmt, param++, resource_buffer->id);
+
 			tracker_db_statement_execute (stmt, &actual_error);
 			g_object_unref (stmt);
 
@@ -612,7 +666,9 @@ tracker_data_resource_buffer_flush (GError **error)
 					g_string_append (fts, g_value_get_string (g_value_array_get_nth (values, i)));
 					g_string_append_c (fts, ' ');
 				}
-				tracker_fts_update_text (resource_buffer->id, tracker_data_query_resource_id (tracker_property_get_uri (prop)), fts->str);
+				tracker_fts_update_text (resource_buffer->id,
+							 tracker_data_query_resource_id (tracker_property_get_uri (prop)),
+							 fts->str, !tracker_property_get_fulltext_no_limit (prop));
 				g_string_free (fts, TRUE);
 			}
 		}
@@ -629,7 +685,6 @@ static void resource_buffer_free (TrackerDataUpdateBufferResource *resource)
 	g_free (resource->subject);
 	resource->subject = NULL;
 
-	g_ptr_array_foreach (resource->types, (GFunc) g_free, NULL);
 	g_ptr_array_free (resource->types, TRUE);
 	resource->types = NULL;
 
@@ -655,6 +710,15 @@ tracker_data_update_buffer_flush (GError **error)
 	resource_buffer = NULL;
 }
 
+void
+tracker_data_update_buffer_might_flush (GError **error)
+{
+	/* avoid high memory usage by update buffer */
+	if (g_hash_table_size (update_buffer.resources) >= 1000) {
+		tracker_data_update_buffer_flush (error);
+	}
+}
+
 static void
 tracker_data_update_buffer_clear (void)
 {
@@ -740,7 +804,6 @@ cache_create_service_decomposed (TrackerClass *cl,
 	TrackerClass       **super_classes;
 	GValue              gvalue = { 0 };
 	gint                i;
-	const gchar        *class_uri;
 
 	/* also create instance of all super classes */
 	super_classes = tracker_class_get_super_classes (cl);
@@ -749,27 +812,28 @@ cache_create_service_decomposed (TrackerClass *cl,
 		super_classes++;
 	}
 
-	class_uri = tracker_class_get_uri (cl);
-
 	for (i = 0; i < resource_buffer->types->len; i++) {
-		if (strcmp (g_ptr_array_index (resource_buffer->types, i), class_uri) == 0) {
+		if (g_ptr_array_index (resource_buffer->types, i) == cl) {
 			/* ignore duplicate statement */
 			return;
 		}
 	}
 
-	g_ptr_array_add (resource_buffer->types, g_strdup (class_uri));
+	g_ptr_array_add (resource_buffer->types, cl);
 
 	g_value_init (&gvalue, G_TYPE_INT);
 
 	cache_insert_row (cl);
 
 	g_value_set_int (&gvalue, ensure_resource_id (tracker_class_get_uri (cl), NULL));
-	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue, TRUE, FALSE);
+	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue, graph, TRUE, FALSE, FALSE);
 
 	tracker_class_set_count (cl, tracker_class_get_count (cl) + 1);
 	if (insert_callbacks) {
 		guint n;
+		const gchar *class_uri;
+
+		class_uri = tracker_class_get_uri (cl);
 
 		for (n = 0; n < insert_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
@@ -802,6 +866,12 @@ value_equal (GValue *value1,
 		/* does RDF define equality for floating point values? */
 		return g_value_get_double (value1) == g_value_get_double (value2);
 	default:
+		if (type == TRACKER_TYPE_DATE_TIME) {
+			/* ignore UTC offset for comparison, irrelevant for comparison according to xsd:dateTime spec
+			 * http://www.w3.org/TR/xmlschema-2/#dateTime
+			 */
+			return tracker_date_time_get_time (value1) == tracker_date_time_get_time (value2);
+		}
 		g_assert_not_reached ();
 	}
 }
@@ -854,8 +924,7 @@ check_property_domain (TrackerProperty *property)
 	gint type_index;
 
 	for (type_index = 0; type_index < resource_buffer->types->len; type_index++) {
-		if (strcmp (tracker_class_get_uri (tracker_property_get_domain (property)),
-		            g_ptr_array_index (resource_buffer->types, type_index)) == 0) {
+		if (tracker_property_get_domain (property) == g_ptr_array_index (resource_buffer->types, type_index)) {
 			return TRUE;
 		}
 	}
@@ -908,6 +977,15 @@ get_property_values (TrackerProperty *property)
 				GValue gvalue = { 0 };
 				_tracker_db_result_set_get_value (result_set, 0, &gvalue);
 				if (G_VALUE_TYPE (&gvalue)) {
+					if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME) {
+						gint time;
+
+						time = g_value_get_int (&gvalue);
+						g_value_unset (&gvalue);
+						g_value_init (&gvalue, TRACKER_TYPE_DATE_TIME);
+						/* UTC offset is irrelevant for comparison */
+						tracker_date_time_set (&gvalue, time, 0);
+					}
 					g_value_array_append (old_values, &gvalue);
 					g_value_unset (&gvalue);
 				}
@@ -953,7 +1031,7 @@ get_old_property_values (TrackerProperty  *property,
 			 */
 			tracker_fts_update_init (resource_buffer->id);
 
-			properties = tracker_ontology_get_properties (&n_props);
+			properties = tracker_ontologies_get_properties (&n_props);
 
 			for (i = 0; i < n_props; i++) {
 				prop = properties[i];
@@ -967,7 +1045,8 @@ get_old_property_values (TrackerProperty  *property,
 					/* delete old fts entries */
 					for (i = 0; i < old_values->n_values; i++) {
 						tracker_fts_update_text (resource_buffer->id, -1,
-						                         g_value_get_string (g_value_array_get_nth (old_values, i)));
+						                         g_value_get_string (g_value_array_get_nth (old_values, i)),
+									 !tracker_property_get_fulltext_no_limit (prop));
 					}
 				}
 			}
@@ -990,7 +1069,8 @@ get_old_property_values (TrackerProperty  *property,
 static void
 string_to_gvalue (const gchar         *value,
                   TrackerPropertyType  type,
-                  GValue              *gvalue)
+                  GValue              *gvalue,
+                  GError             **error)
 {
 	gint object_id;
 
@@ -1015,8 +1095,8 @@ string_to_gvalue (const gchar         *value,
 		break;
 	case TRACKER_PROPERTY_TYPE_DATE:
 	case TRACKER_PROPERTY_TYPE_DATETIME:
-		g_value_init (gvalue, G_TYPE_INT);
-		g_value_set_int (gvalue, tracker_string_to_date (value));
+		g_value_init (gvalue, TRACKER_TYPE_DATE_TIME);
+		tracker_date_time_set_from_string (gvalue, value, error);
 		break;
 	case TRACKER_PROPERTY_TYPE_RESOURCE:
 		object_id = ensure_resource_id (value, NULL);
@@ -1029,24 +1109,28 @@ string_to_gvalue (const gchar         *value,
 	}
 }
 
-static void
+static gboolean
 cache_set_metadata_decomposed (TrackerProperty  *property,
                                const gchar      *value,
+                               const gchar      *graph,
                                GError          **error)
 {
 	gboolean            multiple_values, fts;
 	gchar              *table_name;
 	const gchar        *field_name;
 	TrackerProperty   **super_properties;
-	GValue gvalue = { 0 };
+	GValue              gvalue = { 0 };
 	GValueArray        *old_values;
+	GError             *new_error = NULL;
+	gboolean            change = FALSE;
 
 	/* also insert super property values */
 	super_properties = tracker_property_get_super_properties (property);
 	while (*super_properties) {
-		cache_set_metadata_decomposed (*super_properties, value, error);
-		if (*error) {
-			return;
+		cache_set_metadata_decomposed (*super_properties, value, graph, &new_error);
+		if (new_error) {
+			g_propagate_error (error, new_error);
+			return FALSE;
 		}
 		super_properties++;
 	}
@@ -1063,74 +1147,95 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 
 	fts = tracker_property_get_fulltext_indexed (property);
 
-	/* read existing property values */
-	old_values = get_old_property_values (property, error);
-	if (*error) {
-		g_free (table_name);
-		return;
-	}
-
-	string_to_gvalue (value, tracker_property_get_data_type (property), &gvalue);
-
-	if (!value_set_add_value (old_values, &gvalue)) {
-		/* value already inserted */
-		g_value_unset (&gvalue);
-	} else if (!multiple_values && old_values->n_values > 1) {
-		/* trying to add second value to single valued property */
-
-		g_value_unset (&gvalue);
-
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
-		             "Unable to insert multiple values for subject `%s' and single valued property `%s'",
-		             resource_buffer->subject,
-		             field_name);
-	} else {
-		cache_insert_value (table_name, field_name, &gvalue, multiple_values, fts);
-	}
-
-	g_free (table_name);
-}
-
-static void
-delete_metadata_decomposed (TrackerProperty  *property,
-                            const gchar              *value,
-                            GError          **error)
-{
-	gboolean            multiple_values, fts;
-	gchar              *table_name;
-	const gchar        *field_name;
-	TrackerProperty   **super_properties;
-	GValue gvalue = { 0 };
-	GValueArray        *old_values;
-	GError             *new_error = NULL;
-
-	multiple_values = tracker_property_get_multiple_values (property);
-	if (multiple_values) {
-		table_name = g_strdup_printf ("%s_%s",
-		                              tracker_class_get_name (tracker_property_get_domain (property)),
-		                              tracker_property_get_name (property));
-	} else {
-		table_name = g_strdup (tracker_class_get_name (tracker_property_get_domain (property)));
-	}
-	field_name = tracker_property_get_name (property);
-
-	fts = tracker_property_get_fulltext_indexed (property);
-
 	/* read existing property values */
 	old_values = get_old_property_values (property, &new_error);
 	if (new_error) {
 		g_free (table_name);
 		g_propagate_error (error, new_error);
-		return;
+		return FALSE;
 	}
 
-	string_to_gvalue (value, tracker_property_get_data_type (property), &gvalue);
+	string_to_gvalue (value, tracker_property_get_data_type (property), &gvalue, &new_error);
+	if (new_error) {
+		g_free (table_name);
+		g_propagate_error (error, new_error);
+		return FALSE;
+	}
+
+	if (!value_set_add_value (old_values, &gvalue)) {
+		/* value already inserted */
+		g_value_unset (&gvalue);
+	} else if (!multiple_values && old_values->n_values > 1) {
+		/* trying to add second value to single valued property */
+
+		g_value_unset (&gvalue);
+
+		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
+		             "Unable to insert multiple values for subject `%s' and single valued property `%s'",
+		             resource_buffer->subject,
+		             field_name);
+	} else {
+		cache_insert_value (table_name, field_name, &gvalue, graph, multiple_values, fts,
+		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+
+		change = TRUE;
+	}
+
+	g_free (table_name);
+
+	return change;
+}
+
+static gboolean
+delete_metadata_decomposed (TrackerProperty  *property,
+                            const gchar              *value,
+                            GError          **error)
+{
+	gboolean            multiple_values, fts;
+	gchar              *table_name;
+	const gchar        *field_name;
+	TrackerProperty   **super_properties;
+	GValue gvalue = { 0 };
+	GValueArray        *old_values;
+	GError             *new_error = NULL;
+	gboolean            change = FALSE;
+
+	multiple_values = tracker_property_get_multiple_values (property);
+	if (multiple_values) {
+		table_name = g_strdup_printf ("%s_%s",
+		                              tracker_class_get_name (tracker_property_get_domain (property)),
+		                              tracker_property_get_name (property));
+	} else {
+		table_name = g_strdup (tracker_class_get_name (tracker_property_get_domain (property)));
+	}
+	field_name = tracker_property_get_name (property);
+
+	fts = tracker_property_get_fulltext_indexed (property);
+
+	/* read existing property values */
+	old_values = get_old_property_values (property, &new_error);
+	if (new_error) {
+		g_free (table_name);
+		g_propagate_error (error, new_error);
+		return FALSE;
+	}
+
+	string_to_gvalue (value, tracker_property_get_data_type (property), &gvalue, &new_error);
+
+	if (new_error) {
+		g_free (table_name);
+		g_propagate_error (error, new_error);
+		return FALSE;
+	}
 
 	if (!value_set_remove_value (old_values, &gvalue)) {
 		/* value not found */
 		g_value_unset (&gvalue);
 	} else {
-		cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts);
+		cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts,
+		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+
+		change = TRUE;
 	}
 
 	g_free (table_name);
@@ -1141,6 +1246,8 @@ delete_metadata_decomposed (TrackerProperty  *property,
 		delete_metadata_decomposed (*super_properties, value, error);
 		super_properties++;
 	}
+
+	return change;
 }
 
 static void
@@ -1159,7 +1266,7 @@ cache_delete_resource_type (TrackerClass *class,
 
 	found = FALSE;
 	for (i = 0; i < resource_buffer->types->len; i++) {
-		if (strcmp (g_ptr_array_index (resource_buffer->types, i), tracker_class_get_uri (class)) == 0) {
+		if (g_ptr_array_index (resource_buffer->types, i) == class) {
 			found = TRUE;
 		}
 	}
@@ -1185,7 +1292,7 @@ cache_delete_resource_type (TrackerClass *class,
 			gchar *class_uri;
 
 			tracker_db_result_set_get (result_set, 0, &class_uri, -1);
-			cache_delete_resource_type (tracker_ontology_get_class_by_uri (class_uri), graph);
+			cache_delete_resource_type (tracker_ontologies_get_class_by_uri (class_uri), graph);
 			g_free (class_uri);
 		} while (tracker_db_result_set_iter_next (result_set));
 
@@ -1194,7 +1301,7 @@ cache_delete_resource_type (TrackerClass *class,
 
 	/* delete all property values */
 
-	properties = tracker_ontology_get_properties (&n_props);
+	properties = tracker_ontologies_get_properties (&n_props);
 
 	for (p = 0; p < n_props; p++) {
 		gboolean            multiple_values, fts;
@@ -1232,7 +1339,8 @@ cache_delete_resource_type (TrackerClass *class,
 			g_value_copy (old_gvalue, &gvalue);
 
 			value_set_remove_value (old_values, &gvalue);
-			cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts);
+			cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts,
+			                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
 		}
 
 		g_free (table_name);
@@ -1264,6 +1372,7 @@ tracker_data_delete_statement (const gchar  *graph,
 	TrackerClass       *class;
 	TrackerProperty    *field;
 	gint                subject_id;
+	gboolean change = FALSE;
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
@@ -1296,10 +1405,12 @@ tracker_data_delete_statement (const gchar  *graph,
 	}
 
 	if (object && g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
-		class = tracker_ontology_get_class_by_uri (object);
+		class = tracker_ontologies_get_class_by_uri (object);
 		if (class != NULL) {
 			if (!in_journal_replay) {
-				tracker_db_journal_append_delete_statement_id (resource_buffer->id,
+				tracker_db_journal_append_delete_statement_id (
+					(graph != NULL ? query_resource_id (graph) : 0),
+					resource_buffer->id,
 					tracker_data_query_resource_id (predicate),
 					query_resource_id (object));
 			}
@@ -1310,28 +1421,34 @@ tracker_data_delete_statement (const gchar  *graph,
 			             "Class '%s' not found in the ontology", object);
 		}
 	} else {
-		field = tracker_ontology_get_property_by_uri (predicate);
+		field = tracker_ontologies_get_property_by_uri (predicate);
 		if (field != NULL) {
-			gint id = tracker_property_get_id (field);
-			if (!in_journal_replay) {
+			gint id;
+
+			change = delete_metadata_decomposed (field, object, error);
+
+			id = tracker_property_get_id (field);
+			if (!in_journal_replay && change) {
 				if (tracker_property_get_data_type (field) == TRACKER_PROPERTY_TYPE_RESOURCE) {
-					tracker_db_journal_append_delete_statement_id (resource_buffer->id,
+					tracker_db_journal_append_delete_statement_id (
+						(graph != NULL ? query_resource_id (graph) : 0),
+						resource_buffer->id,
 						(id != 0) ? id : tracker_data_query_resource_id (predicate),
 						query_resource_id (object));
 				} else {
-					tracker_db_journal_append_delete_statement (resource_buffer->id,
+					tracker_db_journal_append_delete_statement (
+						(graph != NULL ? query_resource_id (graph) : 0),
+						resource_buffer->id,
 						(id != 0) ? id : tracker_data_query_resource_id (predicate),
 						object);
 				}
 			}
-
-			delete_metadata_decomposed (field, object, error);
 		} else {
 			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 			             "Property '%s' not found in the ontology", predicate);
 		}
 
-		if (delete_callbacks) {
+		if (delete_callbacks && change) {
 			guint n;
 			for (n = 0; n < delete_callbacks->len; n++) {
 				TrackerStatementDelegate *delegate;
@@ -1415,7 +1532,7 @@ tracker_data_insert_statement_common (const gchar            *graph,
 		                                                 g_free, (GDestroyNotify) cache_table_free);
 
 		g_value_set_int (&gvalue, tracker_data_update_get_next_modseq ());
-		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue, FALSE, FALSE);
+		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue, graph, FALSE, FALSE, FALSE);
 
 		g_hash_table_insert (update_buffer.resources, g_strdup (subject), resource_buffer);
 	}
@@ -1437,7 +1554,7 @@ tracker_data_insert_statement (const gchar            *graph,
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (in_transaction);
 
-	property = tracker_ontology_get_property_by_uri (predicate);
+	property = tracker_ontologies_get_property_by_uri (predicate);
 	if (property != NULL) {
 		if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_RESOURCE) {
 			tracker_data_insert_statement_with_uri (graph, subject, predicate, object, error);
@@ -1463,13 +1580,14 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 	TrackerClass    *class;
 	TrackerProperty *property;
 	gint             prop_id = 0;
+	gboolean change = FALSE;
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (in_transaction);
 
-	property = tracker_ontology_get_property_by_uri (predicate);
+	property = tracker_ontologies_get_property_by_uri (predicate);
 	if (property == NULL) {
 		if (strcmp (predicate, TRACKER_PREFIX "uri") == 0) {
 			/* virtual tracker:uri property */
@@ -1535,26 +1653,31 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 	if (strcmp (predicate, RDF_PREFIX "type") == 0) {
 		/* handle rdf:type statements specially to
 		   cope with inference and insert blank rows */
-		class = tracker_ontology_get_class_by_uri (object);
+		class = tracker_ontologies_get_class_by_uri (object);
 		if (class != NULL) {
 			cache_create_service_decomposed (class, graph);
 		} else {
 			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_CLASS,
 			             "Class '%s' not found in the ontology", object);
+			return;
 		}
+
+		change = TRUE;
 	} else if (strcmp (predicate, TRACKER_PREFIX "uri") == 0) {
 		/* internal property tracker:uri, used to change uri of existing element */
 		resource_buffer->new_subject = g_strdup (object);
+
+		change = TRUE;
 	} else {
 		/* add value to metadata database */
-		cache_set_metadata_decomposed (property, object, &actual_error);
+		change = cache_set_metadata_decomposed (property, object, graph, &actual_error);
 		if (actual_error) {
 			tracker_data_update_buffer_clear ();
 			g_propagate_error (error, actual_error);
 			return;
 		}
 
-		if (insert_callbacks) {
+		if (insert_callbacks && change) {
 			guint n;
 			for (n = 0; n < insert_callbacks->len; n++) {
 				TrackerStatementDelegate *delegate;
@@ -1567,8 +1690,10 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		}
 	}
 
-	if (!in_journal_replay) {
-		tracker_db_journal_append_insert_statement_id (resource_buffer->id,
+	if (!in_journal_replay && change) {
+		tracker_db_journal_append_insert_statement_id (
+			(graph != NULL ? query_resource_id (graph) : 0),
+			resource_buffer->id,
 			(prop_id != 0) ? prop_id : tracker_data_query_resource_id (predicate),
 			query_resource_id (object));
 	}
@@ -1584,13 +1709,15 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 	GError          *actual_error = NULL;
 	TrackerProperty *property;
 	gint             id = 0;
+	gboolean change;
+
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (in_transaction);
 
-	property = tracker_ontology_get_property_by_uri (predicate);
+	property = tracker_ontologies_get_property_by_uri (predicate);
 	if (property == NULL) {
 		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 		             "Property '%s' not found in the ontology", predicate);
@@ -1614,14 +1741,14 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 	}
 
 	/* add value to metadata database */
-	cache_set_metadata_decomposed (property, object, &actual_error);
+	change = cache_set_metadata_decomposed (property, object, graph, &actual_error);
 	if (actual_error) {
 		tracker_data_update_buffer_clear ();
 		g_propagate_error (error, actual_error);
 		return;
 	}
 
-	if (insert_callbacks) {
+	if (insert_callbacks && change) {
 		guint n;
 		for (n = 0; n < insert_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
@@ -1633,8 +1760,10 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 		}
 	}
 
-	if (!in_journal_replay) {
-		tracker_db_journal_append_insert_statement (resource_buffer->id,
+	if (!in_journal_replay && change) {
+		tracker_db_journal_append_insert_statement (
+			(graph != NULL ? query_resource_id (graph) : 0),
+			resource_buffer->id,
 			(id != 0) ? id : tracker_data_query_resource_id (predicate),
 			object);
 	}
@@ -1959,6 +2088,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 	TrackerDBStatement *stmt;
 	TrackerDBResultSet *result_set, *single_result, *multi_result;
 	TrackerClass       *class;
+	gchar              *urn;
 	GString            *sql;
 	TrackerProperty   **properties, *property;
 	int                 i;
@@ -1974,13 +2104,14 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 	/* DROP GRAPH <url> - url here is nie:url */
 
-	stmt = tracker_db_interface_create_statement (iface, "SELECT ID FROM \"nie:DataObject\" WHERE \"nie:DataObject\".\"nie:url\" = ?");
+	stmt = tracker_db_interface_create_statement (iface, "SELECT ID, (SELECT Uri FROM Resource WHERE ID = \"nie:DataObject\".ID) FROM \"nie:DataObject\" WHERE \"nie:DataObject\".\"nie:url\" = ?");
 	tracker_db_statement_bind_text (stmt, 0, url);
 	result_set = tracker_db_statement_execute (stmt, NULL);
 	g_object_unref (stmt);
 
 	if (result_set) {
 		tracker_db_result_set_get (result_set, 0, &resource_id, -1);
+		tracker_db_result_set_get (result_set, 1, &urn, -1);
 		g_object_unref (result_set);
 	} else {
 		/* For fallback to the old behaviour, we could do this here:
@@ -1988,7 +2119,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 		return;
 	}
 
-	properties = tracker_ontology_get_properties (&n_props);
+	properties = tracker_ontologies_get_properties (&n_props);
 
 	stmt = tracker_db_interface_create_statement (iface, "SELECT (SELECT Uri FROM Resource WHERE ID = \"rdf:type\") FROM \"rdfs:Resource_rdf:type\" WHERE ID = ?");
 	tracker_db_statement_bind_int (stmt, 0, resource_id);
@@ -2001,7 +2132,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 			tracker_db_result_set_get (result_set, 0, &class_uri, -1);
 
-			class = tracker_ontology_get_class_by_uri (class_uri);
+			class = tracker_ontologies_get_class_by_uri (class_uri);
 
 			if (class == NULL) {
 				g_warning ("Class '%s' not found in the ontology", class_uri);
@@ -2072,7 +2203,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 					tracker_db_result_set_get (single_result, i++, &value, -1);
 
 					if (value) {
-						tracker_data_delete_statement (graph, url,
+						tracker_data_delete_statement (graph, urn,
 						                               tracker_property_get_uri (property),
 						                               value,
 						                               &new_error);
@@ -2107,7 +2238,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 							tracker_db_result_set_get (multi_result, 0, &value, -1);
 
-							tracker_data_delete_statement (graph, url,
+							tracker_data_delete_statement (graph, urn,
 							                               tracker_property_get_uri (property),
 							                               value,
 							                               &new_error);
@@ -2137,6 +2268,8 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 		g_object_unref (result_set);
 	}
+
+	g_free (urn);
 }
 
 
@@ -2150,6 +2283,12 @@ tracker_data_update_sparql (const gchar  *update,
 
 	g_return_if_fail (update != NULL);
 
+	if (!tracker_db_manager_has_enough_space ()) {
+		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_NO_SPACE,
+			"There is not enough space on the file system for update operations");
+		return;
+	}
+
 	iface = tracker_db_manager_get_db_interface ();
 
 	sparql_query = tracker_sparql_query_new_update (update);
@@ -2198,6 +2337,12 @@ tracker_data_update_sparql_blank (const gchar  *update,
 
 	g_return_val_if_fail (update != NULL, NULL);
 
+	if (!tracker_db_manager_has_enough_space ()) {
+		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_NO_SPACE,
+			"There is not enough space on the file system for update operations");
+		return NULL;
+	}
+
 	iface = tracker_db_manager_get_db_interface ();
 
 	sparql_query = tracker_sparql_query_new_update (update);
