@@ -29,13 +29,14 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <math.h>
 
 #include <glib/gstdio.h>
 
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 #include <libtracker-common/tracker-property.h>
-#include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
 #include "tracker-db-journal.h"
@@ -53,7 +54,7 @@
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_9
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_12
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 
 #define IN_USE_FILENAME               ".meta.isrunning"
@@ -66,15 +67,18 @@ typedef enum {
 
 typedef enum {
 	TRACKER_DB_VERSION_UNKNOWN, /* Unknown */
-	TRACKER_DB_VERSION_1,       /* Version 0.6.6  (before indexer-split) */
-	TRACKER_DB_VERSION_2,       /* Version 0.6.90 (after  indexer-split) */
-	TRACKER_DB_VERSION_3,       /* Version 0.6.91 (stable release) */
-	TRACKER_DB_VERSION_4,       /* Version 0.6.92 (current TRUNK) */
-	TRACKER_DB_VERSION_5,       /* Version 0.7    (vstore branch) */
-	TRACKER_DB_VERSION_6,       /* Version 0.7.4  (nothing special) */
-	TRACKER_DB_VERSION_7,       /* Version 0.7.12 (nmo ontology) */
-	TRACKER_DB_VERSION_8,       /* Version 0.7.13 (coalesce & writeback) */
-	TRACKER_DB_VERSION_9        /* Version 0.7.17 (mlo ontology) */
+	TRACKER_DB_VERSION_01,      /* Version 0.6.6  (before indexer-split) */
+	TRACKER_DB_VERSION_02,      /* Version 0.6.90 (after  indexer-split) */
+	TRACKER_DB_VERSION_03,      /* Version 0.6.91 (stable release) */
+	TRACKER_DB_VERSION_04,      /* Version 0.6.92 (current TRUNK) */
+	TRACKER_DB_VERSION_05,      /* Version 0.7    (vstore branch) */
+	TRACKER_DB_VERSION_06,      /* Version 0.7.4  (nothing special) */
+	TRACKER_DB_VERSION_07,      /* Version 0.7.12 (nmo ontology) */
+	TRACKER_DB_VERSION_08,      /* Version 0.7.13 (coalesce & writeback) */
+	TRACKER_DB_VERSION_09,      /* Version 0.7.17 (mlo ontology) */
+	TRACKER_DB_VERSION_10,      /* Version 0.7.20 (nco im ontology) */
+	TRACKER_DB_VERSION_11,       /* Version 0.7.21 (named graphs/localtime) */
+	TRACKER_DB_VERSION_12       /* Version 0.7.22 (fts-limits branch) */
 } TrackerDBVersion;
 
 typedef struct {
@@ -306,6 +310,47 @@ tracker_db_manager_get_flags (void)
 	return old_flags;
 }
 
+static GValue
+function_sparql_string_join (TrackerDBInterface *interface,
+			     gint                argc,
+			     GValue              values[])
+{
+	GValue result = { 0, };
+	GString *str = NULL;
+	const gchar *separator;
+	gint i;
+
+	/* fn:string-join (str1, str2, ..., separator) */
+
+	if (!G_VALUE_HOLDS_STRING (&values[argc-1])) {
+		/* Fail */
+		return result;
+	}
+
+	separator = g_value_get_string (&values[argc-1]);
+
+	for (i = 0;i < argc-1; i++) {
+		if (G_VALUE_HOLDS_STRING (&values[i])) {
+			if (!str) {
+				str = g_string_new (g_value_get_string (&values[i]));
+			} else {
+				g_string_append_printf (str, "%s%s",
+							separator,
+							g_value_get_string (&values[i]));
+			}
+		}
+	}
+
+	if (str) {
+		g_value_init (&result, G_TYPE_STRING);
+		g_value_set_string (&result, str->str);
+
+		g_string_free (str, TRUE);
+	}
+
+	return result;
+}
+
 /* Create a title-type string from the filename for replacing missing ones */
 static GValue
 function_sparql_string_from_filename (TrackerDBInterface *interface,
@@ -345,6 +390,86 @@ function_sparql_string_from_filename (TrackerDBInterface *interface,
 	return result;
 }
 
+static GValue
+function_sparql_cartesian_distance (TrackerDBInterface *interface,
+				    gint                argc,
+				    GValue              values[])
+{
+	GValue result = { 0, };
+	gdouble lat1;
+	gdouble lat2;
+	gdouble lon1;
+	gdouble lon2;
+
+	gdouble R;
+	gdouble a;
+	gdouble b;
+	gdouble c;
+	gdouble d;
+
+	if (argc != 4) {
+		g_critical ("Invalid argument count");
+		return result;
+	}
+
+	lat1 = g_value_get_double (&values[0])*M_PI/180;
+	lat2 = g_value_get_double (&values[1])*M_PI/180;
+	lon1 = g_value_get_double (&values[2])*M_PI/180;
+	lon2 = g_value_get_double (&values[3])*M_PI/180;
+
+	R = 6371000;
+	a = M_PI/2 - lat1;
+	b = M_PI/2 - lat2;
+	c = sqrt(a*a + b*b - 2*a*b*cos(lon2 - lon1));
+	d = R*c;
+
+	g_value_init (&result, G_TYPE_DOUBLE);
+	g_value_set_double (&result, d);
+
+	return result;
+}
+
+static GValue
+function_sparql_haversine_distance (TrackerDBInterface *interface,
+				    gint                argc,
+				    GValue              values[])
+{
+	GValue result = { 0, };
+	gdouble lat1;
+	gdouble lat2;
+	gdouble lon1;
+	gdouble lon2;
+
+	gdouble R;
+	gdouble dLat;
+	gdouble dLon;
+	gdouble a;
+	gdouble c;
+	gdouble d;
+
+	if (argc != 4) {
+		g_critical ("Invalid argument count");
+		return result;
+	}
+
+	lat1 = g_value_get_double (&values[0])*M_PI/180;
+	lat2 = g_value_get_double (&values[1])*M_PI/180;
+	lon1 = g_value_get_double (&values[2])*M_PI/180;
+	lon2 = g_value_get_double (&values[3])*M_PI/180;
+
+	R = 6371000;
+	dLat = (lat2-lat1);
+	dLon = (lon2-lon1);
+	a = sin(dLat/2) * sin(dLat/2) + cos(lat1) * cos(lat2) *  sin(dLon/2) * sin(dLon/2);
+	c = 2 * atan2(sqrt(a), sqrt(1-a));
+	d = R * c;
+
+	g_value_init (&result, G_TYPE_DOUBLE);
+	g_value_set_double (&result, d);
+
+	return result;
+}
+
 static void
 function_group_concat_step (TrackerDBInterface *interface,
                             void               *aggregate_context,
@@ -734,10 +859,22 @@ db_set_params (TrackerDBInterface *iface,
 		                                             "SparqlRegex",
 		                                             function_sparql_regex,
 		                                             3);
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlStringJoin",
+		                                             function_sparql_string_join,
+		                                             -1);
 		tracker_db_interface_sqlite_create_function (iface,
 		                                             "SparqlStringFromFilename",
 		                                             function_sparql_string_from_filename,
 		                                             1);
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlCartesianDistance",
+		                                             function_sparql_cartesian_distance,
+		                                             4);
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlHaversineDistance",
+		                                             function_sparql_haversine_distance,
+		                                             4);
 		tracker_db_interface_sqlite_create_function (iface,
 		                                             "uncompress",
 		                                             function_uncompress,
@@ -915,17 +1052,21 @@ db_manager_remove_all (gboolean rm_journal)
 	}
 
 	if (rm_journal) {
-		GFile *file;
-		gchar *cpath;
+		const gchar *opath = tracker_db_journal_get_filename ();
 
-		cpath = g_strdup (tracker_db_journal_get_filename ());
-		tracker_db_journal_shutdown ();
-		g_message ("  Removing journal:'%s'",
-		           cpath);
-		file = g_file_new_for_path (cpath);
-		g_file_delete (file, NULL, NULL);
-		g_object_unref (file);
-		g_free (cpath);
+		if (opath) {
+			GFile *file;
+			gchar *cpath;
+
+			cpath = g_strdup (opath);
+			tracker_db_journal_shutdown ();
+			g_message ("  Removing journal:'%s'",
+					   cpath);
+			file = g_file_new_for_path (cpath);
+			g_file_delete (file, NULL, NULL);
+			g_object_unref (file);
+			g_free (cpath);
+		}
 	}
 }
 
@@ -1082,10 +1223,10 @@ db_recreate_all (void)
 	 * we have cleaned up the ontology before we load all
 	 * new databases.
 	 */
-	tracker_ontology_shutdown ();
+	tracker_ontologies_shutdown ();
 
 	/* Make sure we initialize all other modules we depend on */
-	tracker_ontology_init ();
+	tracker_ontologies_init ();
 
 	/* Now create the databases and close them */
 	g_message ("Creating database files, this may take a few moments...");
@@ -1293,14 +1434,14 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 			*first_time = TRUE;
 		}
 
-		if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE)) {
+		if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, TRUE)) {
 			return FALSE;
 		}
 
 		db_recreate_all ();
 
 		/* Make sure we initialize all other modules we depend on */
-		tracker_ontology_init ();
+		tracker_ontologies_init ();
 
 		/* Load databases */
 		g_message ("Loading databases files...");
@@ -1309,7 +1450,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		gboolean must_recreate;
 
 		/* Make sure we initialize all other modules we depend on */
-		tracker_ontology_init ();
+		tracker_ontologies_init ();
 
 		/* Load databases */
 		g_message ("Loading databases files...");
@@ -1373,6 +1514,10 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 				}
 			}
 
+			if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, TRUE)) {
+				return FALSE;
+			}
+
 			db_recreate_all ();
 			loaded = FALSE;
 		}
@@ -1386,7 +1531,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		}
 	}
 
-	in_use_file = g_open (in_use_filename, 
+	in_use_file = g_open (in_use_filename,
 	                      O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
 	                      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
 	fsync (in_use_file);
@@ -1461,7 +1606,7 @@ tracker_db_manager_shutdown (void)
 	db_type_enum_class_pointer = NULL;
 
 	/* Make sure we shutdown all other modules we depend on */
-	tracker_ontology_shutdown ();
+	tracker_ontologies_shutdown ();
 
 	initialized = FALSE;
 	locations_initialized = FALSE;
@@ -1705,3 +1850,16 @@ tracker_db_manager_get_db_interface (void)
 	return resources_iface;
 }
 
+/**
+ * tracker_db_manager_has_enough_space:
+ *
+ * Checks whether the file system, where the database files are stored,
+ * has enough free space to allow modifications.
+ *
+ * returns: TRUE if there is enough space, FALSE otherwise
+ **/
+gboolean
+tracker_db_manager_has_enough_space  (void)
+{
+	return tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, FALSE);
+}
